#include <BasicAbstractions/Button/Button.h>
#include <Configuration.h>
#include <BasicAbstractions/Font.h>

void Button::set(const sf::Vector2f pos, const sf::Vector2f size, const std::string& text, const size_t font_size, std::unique_ptr<ISelectCommand> ptr_command) {
    m_rectangle.setSize(size);
    m_rectangle.setPosition(pos);
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
    m_rectangle.setOutlineThickness(config::BUTTON_FRAME_THICKNESS);
    m_rectangle.setOutlineColor(config::BUTTON_COLOR_FRAME);
    m_text.setFont(MyFont::Instance());
    m_text.setCharacterSize(font_size);
    m_text.setFillColor(config::BUTTON_COLOR_TEXT);
    m_text.setString(text);
    const sf::FloatRect text_bounds = m_text.getLocalBounds();
    const sf::Vector2f button_pos = m_rectangle.getPosition();
    const sf::Vector2f button_size = m_rectangle.getSize();
    m_text.setPosition(
        round(button_pos.x + (button_size.x - text_bounds.width) / 2 - text_bounds.left),
        round(button_pos.y + (button_size.y - text_bounds.height) / 2 - text_bounds.top)
    );
    m_ptr_command = std::move(ptr_command);
}

void Button::select() {
    m_rectangle.setFillColor(config::BUTTON_COLOR_SELECTION);
}

void Button::unselect() {
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
}

bool Button::is_position_in(const sf::Vector2f pos) const noexcept {
    return pos.x >= m_rectangle.getPosition().x && pos.x <= (m_rectangle.getPosition().x + m_rectangle.getSize().x)
        && pos.y >= m_rectangle.getPosition().y && pos.y <= (m_rectangle.getPosition().y + m_rectangle.getSize().y);
}

void Button::push() const {
    m_ptr_command->execute();
}

void Button::draw_into(sf::RenderWindow& window) const {
    window.draw(m_rectangle);
    window.draw(m_text);
}